@@ -28,7 +28,7 @@
 
 #include <Lum/Config/Config.h>
 
-Lum::Model::SizeTRef   unitMode(new Lum::Model::SizeT(1));
+Lum::Model::ULongRef   unitMode(new Lum::Model::ULong(1));
 Lum::Model::BooleanRef prohibitScreenBlanking(new Lum::Model::Boolean(false));
 Lum::Model::BooleanRef compassShowCourseDirection(new Lum::Model::Boolean(false));
 Lum::Model::BooleanRef compassShowRawGPSDirection(new Lum::Model::Boolean(false));
@@ -58,7 +58,7 @@ bool LoadConfig()
   bool prohibitScreenBlanking;
   bool compassShowCourseDirection;
   bool compassShowRawGPSDirection;
-  
+
   if (top->GetAttribute(L"unitMode",unitMode)) {
     ::unitMode->Set(unitMode);
     settingsLoaded++;
@@ -106,12 +106,12 @@ bool SaveConfig()
   }
 
   top->SetAttribute(L"prohibitScreenBlanking",prohibitScreenBlanking->Get());
-  top->SetAttribute(L"unitMode",unitMode->Get());
+  top->SetAttribute(L"unitMode",unitMode->GetUnsignedLong());
   top->SetAttribute(L"compassShowCourseDirection",compassShowCourseDirection->Get());
   top->SetAttribute(L"compassShowRawGPSDirection",compassShowRawGPSDirection->Get());
 
   res=Lum::Config::SaveConfigToXMLFile(path.GetPath(),top);
-  
+
   std::cerr << "SaveConfig to '" << Lum::Base::WStringToString(path.GetPath()) << "': " << (res ? "OK" : "ERROR") << std::endl;
 
   delete top;
