@@ -27,6 +27,8 @@
 
 #include <cerrno>
 #include <cmath>
+#include <iomanip>
+#include <iostream>
 
 #if defined (HAVE_LIB_GPS)
   #include <gps.h>
@@ -89,55 +91,16 @@ extern "C" {
 #include "Configuration.h"
 
 #include "GPSData.h"
+#include "Tracks.h"
 #include "Compass.h"
 #include "Speed.h"
 #include "TimeLine.h"
 #include "Util.h"
 
-#include <iostream>
-
 static long reconnectTime   = 10; // Period of time after which we check for disconnection and
                                   // trigger reconnection
 static long refreshTime     = 10; // Period of time after which we update the stream views
 
-std::wstring SystemTimeToISO8601(const Lum::Base::SystemTime& time)
-{
-  Lum::Base::DateTime t;
-  std::wstring        res;
-
-  res.reserve(10+1+8+1);
-
-  time.GetUTCDateTime(t);
-
-  // Date
-  res.append(1,(wchar_t)(L'0'+(t.year / 1000) % 10));
-  res.append(1,(wchar_t)(L'0'+(t.year /  100) % 10));
-  res.append(1,(wchar_t)(L'0'+(t.year /   10) % 10));
-  res.append(1,(wchar_t)(L'0'+(t.year /    1) % 10));
-  res.append(L"-");
-  res.append(1,(wchar_t)(L'0'+(t.month / 10) % 10));
-  res.append(1,(wchar_t)(L'0'+(t.month /  1) % 10));
-  res.append(L"-");
-  res.append(1,(wchar_t)(L'0'+(t.dayOfMonth / 10) % 10));
-  res.append(1,(wchar_t)(L'0'+(t.dayOfMonth /  1) % 10));
-
-  res.append(1,'T');
-
-  // Time
-  res.append(1,(wchar_t)(L'0'+(t.hour /   10) % 10));
-  res.append(1,(wchar_t)(L'0'+(t.hour /    1) % 10));
-  res.append(L":");
-  res.append(1,(wchar_t)(L'0'+(t.minute / 10) % 10));
-  res.append(1,(wchar_t)(L'0'+(t.minute /  1) % 10));
-  res.append(L":");
-  res.append(1,(wchar_t)(L'0'+(t.second / 10) % 10));
-  res.append(1,(wchar_t)(L'0'+(t.second /  1) % 10));
-
-  res.append(1,'Z');
-
-  return res;
-}
-
 typedef Lum::Model::StdTable<GPSData::Satellite> SatellitesModel;
 typedef Lum::Base::Reference<SatellitesModel>    SatellitesModelRef;
 
@@ -154,6 +117,25 @@ double GetRelativeQualityFromDbm(double quality)
   }
 }
 
+class SatellitesComparator : public SatellitesModel::Comparator
+{
+public:
+  bool operator()(const GPSData::Satellite& a,
+                  const GPSData::Satellite& b,
+                  size_t column,
+                  bool ascending) const
+  {
+    assert(column==1);
+
+    if (ascending) {
+      return a.snr<b.snr;
+    }
+    else {
+      return a.snr>b.snr;
+    }
+  }
+};
+
 class SatellitesModelPainter : public Lum::TableCellPainter
 {
 private:
@@ -235,10 +217,9 @@ public:
 };
 
 class MyWindow;
+
 static MyWindow*         window=NULL;
 static Lum::Def::AppInfo info;
-static unsigned long     trackingFileId='G'*256*256*256+'J'*256*256+'T'*256+'\0';
-static unsigned long     trackingFileVersion=2;
 
 #if defined(HAVE_LIB_GPS)
   #if GPSD_API_MAJOR_VERSION>=4
@@ -260,39 +241,6 @@ static void gps_location_stopped(LocationGPSDControl *control, gpointer userdata
 static void gps_location_error(LocationGPSDControl *control, gpointer userdata);
 #endif
 
-class Track
-{
-private:
-  std::wstring          name;
-  Lum::Base::SystemTime time;
-
-public:
-  Track()
-  {
-    // no code
-  }
-
-  Track(const std::wstring& name, const Lum::Base::SystemTime& time)
-  : name(name),
-    time(time)
-  {
-    // no code
-  }
-
-  std::wstring GetName() const
-  {
-    return name;
-  }
-
-  std::wstring GetTimeString() const
-  {
-    return time.GetLocalLocaleDate()+L" "+time.GetLocalLocaleTime();
-  }
-};
-
-typedef Lum::Model::StdTable<Track> TracksModel;
-typedef Lum::Base::Reference<TracksModel> TracksModelRef;
-
 class TracksModelPainter : public Lum::TableCellPainter
 {
 public:
@@ -304,8 +252,6 @@ public:
     //
     // Name of Track
     //
-
-
     DrawStringLeftAligned(draw,
                            x,y,width,height,
                            track.GetName());
@@ -313,7 +259,6 @@ public:
     //
     //
     // Time of track
-
     DrawStringRightAligned(draw,
                            x,y,width,height,
                            track.GetTimeString());
@@ -323,11 +268,11 @@ public:
 class TrackComparator : public TracksModel::Comparator
 {
 public:
-  bool operator()(const Track& a, const Track& b, size_t column, bool down) const
+  bool operator()(const Track& a, const Track& b, size_t column, bool ascending) const
   {
     assert(column==1);
 
-    if (down) {
+    if (ascending) {
       return a.GetName()<b.GetName();
     }
     else {
@@ -360,7 +305,8 @@ private:
   Lum::Model::ActionRef    startTrackingAction;
   Lum::Model::ActionRef    stopTrackingAction;
   Lum::Model::ActionRef    deleteTrackAction;
-  Lum::Model::ActionRef    exportTrackAction;
+  Lum::Model::ActionRef    exportTrackKMLAction;
+  Lum::Model::ActionRef    exportTrackGPXAction;
   Lum::Model::ActionRef    settingsAction;
   Lum::Model::ActionRef    aboutAction;
 
@@ -369,8 +315,13 @@ private:
   Lum::Model::ActionRef    reconnectCheckTimer;
 
   Lum::Model::BooleanRef   tracking;
+  Lum::Model::StringRef    trackingStartDateTime;
+  Lum::Model::StringRef    trackingDataPoints;
+  Lum::Model::StringRef    trackingFileSize;
 
   std::ofstream            trackFile;
+  unsigned long            trackingCount;
+  unsigned long            trackingSize;
 
 #if defined(HAVE_LIB_LOCATION)
   LocationGPSDevice        *device;
@@ -533,7 +484,7 @@ private:
       gps_set_callback(gpsData,cb,&gpsThread);
 #endif
       data.SetConnected(true);
-      std::cout << "GPS initialized!" << std::endl;
+      std::cout << "GPS reinitialized!" << std::endl;
     }
     else {
       data.SetConnected(false);
@@ -556,7 +507,7 @@ public:
     track(new Lum::Model::String(L"")),
     direction(new Lum::Model::Double()),
     climb(new Lum::Model::String(L"")),
-    satellites(new SatellitesModel()),
+    satellites(new SatellitesModel(new SatellitesComparator())),
     speedStream(new Lum::Model::DoubleDataStream()),
     altitudeStream(new Lum::Model::DoubleDataStream()),
     tracks(new TracksModel(new TrackComparator())),
@@ -564,13 +515,17 @@ public:
     startTrackingAction(new Lum::Model::Action()),
     stopTrackingAction(new Lum::Model::Action()),
     deleteTrackAction(new Lum::Model::Action()),
-    exportTrackAction(new Lum::Model::Action()),
+    exportTrackKMLAction(new Lum::Model::Action()),
+    exportTrackGPXAction(new Lum::Model::Action()),
     settingsAction(new Lum::Model::Action()),
     aboutAction(new Lum::Model::Action()),
     dataChangedAction(new Lum::Model::Action()),
     refreshTimer(new Lum::Model::Action()),
     reconnectCheckTimer(new Lum::Model::Action()),
-    tracking(new Lum::Model::Boolean(false))
+    tracking(new Lum::Model::Boolean(false)),
+    trackingStartDateTime(new Lum::Model::String()),
+    trackingDataPoints(new Lum::Model::String()),
+    trackingFileSize(new Lum::Model::String())
 #if defined(HAVE_LIB_LOCATION)
     ,device(NULL)
     ,control(NULL)
@@ -591,7 +546,8 @@ public:
     Observe(startTrackingAction);
     Observe(stopTrackingAction);
     Observe(deleteTrackAction);
-    Observe(exportTrackAction);
+    Observe(exportTrackKMLAction);
+    Observe(exportTrackGPXAction);
     Observe(settingsAction);
     Observe(aboutAction);
 
@@ -699,6 +655,26 @@ public:
                                                 ->AddSpace()
                                                 ->Add(Lum::View::Create(TimeLine::Create(altitudeStream,true,true),true,true))));
 
+    multiView.AddView(Lum::Def::MultiView::View(5,
+                                                Lum::Def::Desc(L"Recording"),
+                                                Lum::VPanel::Create(true,true)
+                                                ->Add(Lum::HPanel::Create(true,false)
+                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Record"),
+                                                                                                                                   startTrackingAction)
+                                                                                                                  .SetImage(L"Record.png")))
+                                                      ->AddSpace(true)
+                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Stop"),
+                                                                                                                                   stopTrackingAction)
+                                                                                                                  .SetImage(L"Stop.png"))))
+                                                ->AddSpace(false)
+                                                ->Add(Lum::Label::Create(true,false)
+                                                      ->AddLabel(L"StartTime:",Lum::TextValue::Create(trackingStartDateTime,true))
+                                                      ->AddLabel(L"Data points:",Lum::TextValue::Create(trackingDataPoints,true))
+                                                      ->AddLabel(L"File size:",Lum::TextValue::Create(trackingFileSize,true))
+                                                      )
+                                                ->AddSpace(true)
+                                               ));
+
     headerModel=new Lum::Model::HeaderImpl();
     headerModel->AddColumn(L"Track",Lum::Base::Size::stdCharWidth,40,true);
 
@@ -711,25 +687,20 @@ public:
     table->SetPainter(new TracksModelPainter());
     table->GetTableView()->SetAutoFitColumns(true);
 
-    multiView.AddView(Lum::Def::MultiView::View(5,
-                                                Lum::Def::Desc(L"Tracking"),
+    multiView.AddView(Lum::Def::MultiView::View(6,
+                                                Lum::Def::Desc(L"Tracks"),
                                                 Lum::VPanel::Create(true,true)
                                                 ->Add(Lum::HPanel::Create(true,false)
-                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Record"),
-                                                                                                                                   startTrackingAction)
-                                                                                                                  .SetImage(L"Record.png")))
-                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Stop"),
-                                                                                                                                   stopTrackingAction)
-                                                                                                                  .SetImage(L"Stop.png")))
-                                                      ->AddSpace(true)
-                                                       ->Add(new Lum::LED(tracking))
-                                                       ->AddSpace(true)
                                                       ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Delete"),
                                                                                                                                    deleteTrackAction)
                                                                                                                   .SetImage(L"Delete.png")))
-                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Export"),
-                                                                                                                                   exportTrackAction)
-                                                                                                                  .SetImage(L"Save.png"))))
+                                                      ->AddSpace(true)
+                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Export KML"),
+                                                                                                                                   exportTrackKMLAction)
+                                                                                                                  .SetImage(L"Save_KML.png")))
+                                                      ->Add(Lum::Manager::Behaviour::Instance()->GetActionControl(Lum::Def::Action(Lum::Def::Desc(L"_Export GPX"),
+                                                                                                                                   exportTrackGPXAction)
+                                                                                                                  .SetImage(L"Save_GPX.png"))))
                                                 ->AddSpace()
                                                 ->Add(table)));
 
@@ -767,67 +738,18 @@ public:
 
   void StartTracking()
   {
-    Lum::Model::StringRef trackName=new Lum::Model::String(L"");
-    std::wstring          homeDir;
-    Lum::Base::Path       fileName;
+    if (::StartTracking(GetWindow(),trackFile)) {
+      Lum::Base::SystemTime now;
 
-    if (!Lum::Dlg::TextInput::GetText(GetWindow(),
-                                      L"Enter track name!",
-                                      L"Please enter a name for the to be recorded track.",
-                                      trackName) || trackName->Empty()) {
-      return;
+      trackingCount=0;
+      trackingSize=0;
+      trackingStartDateTime->Set(now.GetLocalLocaleDateTime());
+      trackingDataPoints->Set(L"0");
+      trackingFileSize->Set(Lum::Base::NumberToWString(GetFileSize(trackFile)));
+      startTrackingAction->Disable();
+      stopTrackingAction->Enable();
+      tracking->Set(true);
     }
-
-    if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
-        homeDir)) {
-      return;
-    }
-
-    fileName.SetNativeDir(homeDir);
-    fileName.AppendDir(L".gpsjinni");
-
-    Lum::Base::Path::CreateDir(fileName.GetPath());
-
-    fileName.SetBaseName(trackName->Get()+L".gjt");
-
-    if (Lum::Base::Path::Exists(fileName.GetPath())) {
-      Lum::Dlg::Msg::ShowOk(GetWindow(),
-                            L"Error while opening file!",
-                            std::wstring(L"An error occured while opening file:\n")+
-                            L"'"+fileName.GetPath()+L"'\n"+
-                            L"Error:\n"+
-                            L"'File already exists'");
-      return;
-    }
-
-    trackFile.open(Lum::Base::WStringToString(fileName.GetPath()).c_str(),
-                   std::ios::out|std::ios::trunc|std::ios::binary);
-
-    if (!trackFile) {
-      Lum::Base::Status status;
-
-      status.SetToCurrentErrno();
-      Lum::Dlg::Msg::ShowOk(GetWindow(),
-                            L"Error while opening file!",
-                            std::wstring(L"An error occured while opening file:\n")+
-                            L"'"+fileName.GetPath()+L"'\n"+
-                            L"Error:\n"+
-                            L"'"+status.GetDescription()+L"'");
-
-      return;
-    }
-
-    unsigned long tfid=htonl(trackingFileId);
-    unsigned long tfv=htonl(trackingFileVersion);
-    unsigned long tft=htonl(Lum::Base::SystemTime().GetTime());
-
-    trackFile.write((const char*)&tfid,sizeof(tfid));
-    trackFile.write((const char*)&tfv,sizeof(tfv));
-    trackFile.write((const char*)&tft,sizeof(tft));
-
-    startTrackingAction->Disable();
-    stopTrackingAction->Enable();
-    tracking->Set(true);
   }
 
   void AddTrackingRecord(const GPSData& data)
@@ -836,42 +758,14 @@ public:
       return;
     }
 
-    if (!data.HasFix()) {
-      return;
+    if (::AddTrackingRecord(trackFile,data)) {
+      trackingCount++;
+      trackingDataPoints->Set(Lum::Base::NumberToWString(trackingCount));
+      trackingFileSize->Set(Lum::Base::NumberToWString(GetFileSize(trackFile)));
     }
-
-    double                lon=0.0;
-    double                lat=0.0;
-    double                alt=0.0;
-    Lum::Base::SystemTime tim;
-
-    if (data.HasLongitude()) {
-      lon=data.GetLongitude();
-    }
-
-    if (data.HasLatitude()) {
-      lat=data.GetLatitude();
-    }
-
-    if (data.HasAltitude()) {
-      alt=data.GetAltitude();
-    }
-
-    if (data.HasTime()) {
-      tim=data.GetTime();
-    }
-
-    unsigned long flon=htonl((unsigned long)rint((lon+180.0)*10000000.0));
-    unsigned long flat=htonl((unsigned long)rint((lat+90.0)*10000000.0));
-    unsigned long falt=htonl((unsigned long)rint(alt*100));
-    unsigned long ftim=htonl(tim.GetTime());
-
-    trackFile.write((const char*)&flon,sizeof(flon));
-    trackFile.write((const char*)&flat,sizeof(flat));
-    trackFile.write((const char*)&falt,sizeof(falt));
-    trackFile.write((const char*)&ftim,sizeof(ftim));
   }
 
+
   void StopTracking()
   {
     trackFile.close();
@@ -880,435 +774,11 @@ public:
     stopTrackingAction->Disable();
     tracking->Set(false);
 
-    RefreshTracks();
-  }
+    trackingStartDateTime->SetNull();
+    trackingDataPoints->SetNull();
+    trackingFileSize->SetNull();
 
-  void RefreshTracks()
-  {
-    Lum::Base::Path              dirName,fileName;
-    Lum::Base::DirScannerRef     scanner;
-    Lum::Base::DirScanner::Entry entry;
-    Lum::Base::Status            status;
-    std::wstring                 homeDir;
-
-    tracks->Clear();
-    tracks->Off();
-
-    if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
-        homeDir)) {
-      return;
-    }
-
-    dirName.SetNativeDir(homeDir);
-    dirName.AppendDir(L".gpsjinni");
-
-    scanner=Lum::Base::DirScanner::Create(dirName);
-
-    while (scanner->GetNext(entry,status)) {
-      if (entry.isDir) {
-        continue;
-      }
-
-      std::cout << "Found possible track '" << Lum::Base::WStringToString(entry.name) << "'" << std::endl;
-
-      if (entry.name.length()<4 ||
-          entry.name.substr(entry.name.length()-4)!=L".gjt") {
-        std::cerr << "wrong suffix!" << std::endl;
-        continue;
-      }
-
-      std::ifstream file;
-
-      fileName=dirName;
-      fileName.SetBaseName(entry.name);
-
-      file.open(Lum::Base::WStringToString(fileName.GetPath()).c_str(),
-                std::ios::in|std::ios::binary);
-
-      if (!file) {
-        std::cerr << "Cannot open file!" << std::endl;
-        continue;
-      }
-
-      unsigned long tfid,tfv,tft;
-
-      file.read((char*)&tfid,sizeof(tfid));
-      file.read((char*)&tfv,sizeof(tfv));
-      file.read((char*)&tft,sizeof(tft));
-
-      if (!file) {
-        std::cerr << "Cannot read file header!" << std::endl;
-        continue;
-      }
-
-      tfid=ntohl(tfid);
-      tfv=ntohl(tfv);
-      tft=ntohl(tft);
-
-      if (tfid!=trackingFileId) {
-        std::cerr << "Wrong magic!" << std::endl;
-        continue;
-      }
-
-      /* We support the current version and version 1 */
-      if (tfv!=trackingFileVersion && tfv!=1) {
-        std::cerr << "Wrong file version!" << std::endl;
-        continue;
-      }
-
-      tracks->Append(Track(entry.name.substr(0,entry.name.length()-4),
-                           Lum::Base::SystemTime(tft)));
-    }
-
-    tracks->Sort(1);
-
-    tracks->On();
-  }
-
-  struct DataPoint
-  {
-    Lum::Base::SystemTime time;
-    double                lat;
-    double                lon;
-    double                alt;
-  };
-
-  void ExportTrack(const std::wstring& name)
-  {
-    Lum::Base::Path                path;
-    std::wstring                   file;
-    Lum::Dlg::File::Options        *options=new Lum::Dlg::File::Options();
-    Lum::Model::Dir::PatternFilter *filter;
-    std::ifstream                  in;
-    std::ofstream                  out;
-    std::wstring                   homeDir;
-
-    if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
-        homeDir)) {
-      return;
-    }
-
-    path.SetNativeDir(homeDir);
-    path.AppendDir(L".gpsjinni");
-    path.SetBaseName(name+L".gjt");
-
-    in.open(Lum::Base::WStringToString(path.GetPath()).c_str(),
-            std::ios::in|std::ios::binary);
-
-    if (!in) {
-      Lum::Base::Status status;
-
-      status.SetToCurrentErrno();
-      Lum::Dlg::Msg::ShowOk(GetWindow(),
-                            L"Error while opening file!",
-                            std::wstring(L"An error occured while opening file:\n")+
-                            L"'"+file+L"'\n"+
-                            L"Error:\n"+
-                            L"'"+status.GetDescription()+L"'");
-
-      return;
-    }
-
-    unsigned long tfid,tfv,tft;
-
-    in.read((char*)&tfid,sizeof(tfid));
-    in.read((char*)&tfv,sizeof(tfv));
-    in.read((char*)&tft,sizeof(tft));
-
-    if (!in) {
-      std::cerr << "Cannot read file header!" << std::endl;
-      return;
-    }
-
-    tfid=ntohl(tfid);
-    tfv=ntohl(tfv);
-    tft=ntohl(tft);
-
-    if (tfid!=trackingFileId) {
-      Lum::Dlg::Msg::ShowOk(GetWindow(),
-                            L"Error while opening file!",
-                            std::wstring(L"An error occured while opening file:\n")+
-                            L"'"+file+L"'\n"+
-                            L"Error:\n"+
-                            L"'File is not a stored track'");
-      return;
-    }
-
-    /* We support the current version and version 1 */
-    if (tfv!=trackingFileVersion && tfv!=1) {
-      Lum::Dlg::Msg::ShowOk(GetWindow(),
-                            L"Error while opening file!",
-                            std::wstring(L"An error occured while opening file:\n")+
-                            L"'"+file+L"'\n"+
-                            L"Error:\n"+
-                            L"'File format version not supported'");
-      return;
-    }
-
-    std::cout << "Track data: " << std::hex << tfid << std::dec << " " << tfv << " " << tft << std::endl;
-
-    std::wstring docsDir;
-
-    if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDocumentDir,
-                                                        docsDir)) {
-      if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
-                                                          docsDir)) {
-        return;
-      }
-    }
-
-    path.SetNativeDirAndFile(docsDir,name+L".kml");
-
-    file=path.GetPath();
-
-    options->SetMode(Lum::Dlg::File::Options::modeFile);
-    options->SetExistingOnly(false);
-
-    filter=new Lum::Model::Dir::PatternFilter(L"KML Files");
-    filter->AddPattern(L"*.kml");
-    options->AddFilter(filter);
-
-    if (!Lum::Dlg::File::SaveFile(GetWindow(),
-                                  L"Exporting track...",
-                                  file,
-                                  options)) {
-      return;
-    }
-
-    out.open(Lum::Base::WStringToString(file).c_str(),
-             std::ios::out|std::ios::trunc);
-
-    if (!out) {
-      Lum::Base::Status status;
-
-      status.SetToCurrentErrno();
-      Lum::Dlg::Msg::ShowOk(GetWindow(),
-                            L"Error while opening file!",
-                            std::wstring(L"An error occured while opening file:\n")+
-                            L"'"+file+L"'\n"+
-                            L"Error:\n"+
-                            L"'"+status.GetDescription()+L"'");
-
-      return;
-    }
-
-    std::list<DataPoint> data;
-
-    while (in) {
-      unsigned long flon;
-      unsigned long flat;
-      unsigned long falt;
-      unsigned long ftim;
-
-      in.read((char*)&flon,sizeof(flon));
-      in.read((char*)&flat,sizeof(flat));
-      in.read((char*)&falt,sizeof(falt));
-      in.read((char*)&ftim,sizeof(ftim));
-
-      if (in) {
-        DataPoint dataPoint;
-
-        flon=ntohl(flon);
-        flat=ntohl(flat);
-        falt=ntohl(falt);
-        ftim=ntohl(ftim);
-
-        //std::cout << flon/10000000.0 << " " << flat/10000000.0 << " " << falt/100.0 << " " << ftim << std::endl;
-
-        if (tfv==1) {
-          dataPoint.lon=flon/10000000.0-90.0;
-          dataPoint.lat=flat/10000000.0-180.0;
-        }
-        else {
-          dataPoint.lon=flon/10000000.0-180.0;
-          dataPoint.lat=flat/10000000.0-90.0;
-        }
-        dataPoint.alt=falt/100.0;
-        dataPoint.time.SetTime(ftim);
-
-        data.push_back(dataPoint);
-      }
-    }
-
-    out << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << std::endl;
-    out << "<kml xmlns=\"http://www.opengis.net/kml/2.2\">" << std::endl;
-    out << "  <Document>" << std::endl;
-    out << "    <name>" << Lum::Base::WStringToUTF8(name) << ".kml" << "</name>" << std::endl;
-    out << "    <open>1</open>" << std::endl;
-    out << std::endl;
-    out << "    <Style id=\"startpointStyle\">" << std::endl;
-    out << "      <IconStyle>" << std::endl;
-    out << "        <Icon>" << std::endl;
-    out << "          <href>http://maps.google.com/mapfiles/kml/paddle/A.png</href>" << std::endl;
-    out << "        </Icon>" << std::endl;
-    out << "      </IconStyle>" << std::endl;
-    out << "    </Style>" << std::endl;
-    out << std::endl;
-    out << "    <Style id=\"waypointStyle\">" << std::endl;
-    out << "      <IconStyle>" << std::endl;
-    out << "        <Icon>" << std::endl;
-    out << "          <href>http://maps.google.com/mapfiles/kml/pushpin/ylw-pushpin.png</href>" << std::endl;
-    out << "        </Icon>" << std::endl;
-    out << "      </IconStyle>" << std::endl;
-    out << "    </Style>" << std::endl;
-    out << std::endl;
-    out << "    <Style id=\"endpointStyle\">" << std::endl;
-    out << "      <IconStyle>" << std::endl;
-    out << "        <Icon>" << std::endl;
-    out << "          <href>http://maps.google.com/mapfiles/kml/paddle/B.png</href>" << std::endl;
-    out << "        </Icon>" << std::endl;
-    out << "      </IconStyle>" << std::endl;
-    out << "    </Style>" << std::endl;
-    out << std::endl;
-
-    out << "    <Folder>" << std::endl;
-    out << "      <name>" << Lum::Base::WStringToUTF8(name) << " waypoints with timestamps</name>" << std::endl;
-    out << "      <open>1</open>" << std::endl;
-    out << std::endl;
-    out << "      <Style>" << std::endl;
-    out << "        <ListStyle>" << std::endl;
-    out << "          <listItemType>checkHideChildren</listItemType>" << std::endl;
-    out << "        </ListStyle>" << std::endl;
-    out << "      </Style>" << std::endl;
-
-    for (std::list<DataPoint>::const_iterator dataPoint=data.begin();
-         dataPoint!=data.end();
-         ++dataPoint) {
-      out << std::endl;
-      out << "      <Placemark>" << std::endl;
-      out << "        <TimeStamp>" << std::endl;
-      out << "          <when>" << Lum::Base::WStringToString(SystemTimeToISO8601(dataPoint->time)) << "</when>" << std::endl;
-      out << "        </TimeStamp>" << std::endl;
-      out << "        <styleUrl>#waypointStyle</styleUrl>" << std::endl;
-      out << "        <Point>" << std::endl;
-      out << "          <coordinates>" << dataPoint->lon << "," << dataPoint->lat << "," << dataPoint->alt << "</coordinates>" << std::endl;
-      out << "        </Point>" << std::endl;
-      out << "      </Placemark>" << std::endl;
-    }
-    out << "    </Folder>" << std::endl;
-
-    out << std::endl;
-    out << std::endl;
-
-    out << "    <Folder>" << std::endl;
-    out << "      <name>" << Lum::Base::WStringToUTF8(name) << " path waypoints</name>" << std::endl;
-    out << "      <open>1</open>" << std::endl;
-    out << std::endl;
-    out << "      <Style>" << std::endl;
-    out << "        <ListStyle>" << std::endl;
-    out << "          <listItemType>checkHideChildren</listItemType>" << std::endl;
-    out << "        </ListStyle>" << std::endl;
-    out << "      </Style>" << std::endl;
-
-    for (std::list<DataPoint>::const_iterator dataPoint=data.begin();
-         dataPoint!=data.end();
-         ++dataPoint) {
-      out << std::endl;
-      out << "      <Placemark>" << std::endl;
-      out << "        <styleUrl>#waypointStyle</styleUrl>" << std::endl;
-      out << "        <Point>" << std::endl;
-      out << "          <coordinates>" << dataPoint->lon << "," << dataPoint->lat << "," << dataPoint->alt << "</coordinates>" << std::endl;
-      out << "        </Point>" << std::endl;
-      out << "      </Placemark>" << std::endl;
-    }
-    out << "    </Folder>" << std::endl;
-
-    out << std::endl;
-    out << std::endl;
-
-    if (data.size()>0) {
-      out << "    <Placemark>" << std::endl;
-      out << "      <styleUrl>#startpointStyle</styleUrl>" << std::endl;
-      out << "      <name>Start</name>" << std::endl;
-      out << "      <Point>" << std::endl;
-      out << "        <coordinates>" << data.begin()->lon << "," << data.begin()->lat << "," << data.begin()->alt << "</coordinates>" << std::endl;
-      out << "      </Point>" << std::endl;
-      out << "    </Placemark>" << std::endl;
-      out << std::endl;
-    }
-
-    if (data.size()>0) {
-      out << "    <Placemark>" << std::endl;
-      out << "      <styleUrl>#endpointStyle</styleUrl>" << std::endl;
-      out << "      <name>End</name>" << std::endl;
-      out << "      <Point>" << std::endl;
-      out << "        <coordinates>" << data.rbegin()->lon << "," << data.rbegin()->lat << "," << data.rbegin()->alt << "</coordinates>" << std::endl;
-      out << "      </Point>" << std::endl;
-      out << "    </Placemark>" << std::endl;
-      out << std::endl;
-    }
-
-    out << "    <Placemark>" << std::endl;
-    out << "      <name>" << Lum::Base::WStringToUTF8(name) << " path</name>" << std::endl;
-    out << std::endl;
-    out << "      <Style>" << std::endl;
-    out << "        <LineStyle>" << std::endl;
-    out << "          <color>ff0000ff</color>" << std::endl;
-    out << "          <width>2</width>" << std::endl;
-    out << "        </LineStyle>" << std::endl;
-    out << "      </Style>" << std::endl;
-    out << std::endl;
-    out << "      <LineString>" << std::endl;
-    //out << "        <extrude>0</extrude>" << std::endl;
-    out << "        <tessellate>1</tessellate>" << std::endl;
-    out << "        <altitudeMode>clampToGround</altitudeMode>" << std::endl;
-    out << "        <coordinates>" << std::endl;
-
-    double lastLat=1000;
-    double lastLon=1000;
-    double lastAlt=1000;
-
-    for (std::list<DataPoint>::const_iterator dataPoint=data.begin();
-         dataPoint!=data.end();
-         ++dataPoint) {
-      if (dataPoint->lon!=lastLon ||
-          dataPoint->lat!=lastLat ||
-          dataPoint->alt!=lastAlt) {
-        out << "          " << dataPoint->lon << "," << dataPoint->lat << "," << dataPoint->alt << std::endl;
-        lastLon=dataPoint->lon;
-        lastLat=dataPoint->lat;
-        lastAlt=dataPoint->alt;
-      }
-    }
-
-    out << "        </coordinates>" << std::endl;
-    out << "      </LineString>" << std::endl;
-    out << "    </Placemark>" << std::endl;
-    out << "  </Document>" << std::endl;
-    out << "</kml>" << std::endl;
-
-    out.close();
-    in.close();
-  }
-
-  void DeleteTrack(const std::wstring& name)
-  {
-    Lum::Base::Path   path;
-    Lum::Base::Status status;
-    std::wstring      homeDir;
-
-    if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
-                                                        homeDir)) {
-      return;
-    }
-
-    path.SetNativeDir(homeDir);
-    path.AppendDir(L".gpsjinni");
-    path.SetBaseName(name+L".gjt");
-
-    status=Lum::Base::Path::RemoveFile(path.GetPath());
-
-    if (status) {
-      RefreshTracks();
-    }
-    else {
-      Lum::Dlg::Msg::ShowOk(GetWindow(),
-                            L"Error while deleting file!",
-                            std::wstring(L"An error occured while deletion of file:\n")+
-                            L"'"+path.GetPath()+L"'\n"+
-                            L"Error:\n"+
-                            L"'"+status.GetDescription()+L"'");
-    }
+    RefreshTracks(tracks);
   }
 
   void SetCompassDirectionRaw(double newDirection, double newDeviation)
@@ -1329,30 +799,28 @@ public:
     if (isnan(newDirection) ||
         isnan(newDeviation) ||
         direction->IsNull() ||
-        isnan(oldDirection = direction->Get())
-        ) {
-      SetCompassDirectionRaw(newDirection, newDeviation);
+        isnan(oldDirection = direction->GetDouble())) {
+      SetCompassDirectionRaw(newDirection,newDeviation);
       return;
     }
 
-    if (newDeviation < 0) {     // just in case...
-      newDeviation = -newDeviation;
+    if (newDeviation<0) {     // just in case...
+      newDeviation=-newDeviation;
     }
 
     // Ignore new direction if deviation is too high; leave old direction valid.
 
-    if (newDeviation > 180) {
+    if (newDeviation>180) {
       return;
     }
 
     // Weighten the new direction by its accuracy (that is inverse deviation)
     // in comparison to the old direction is a first experiment...
     // TODO: find better methods to find and eliminate wild shot values.
-    newDirection = CalculateWeightedDirection(
-                      oldDirection,
-                      newDirection,
-                      1 - (newDeviation / 360),    // normalized weight in range 0..1.,
-                      360);
+    newDirection=CalculateWeightedDirection(oldDirection,
+                                            newDirection,
+                                            1-(newDeviation/360), // normalized weight in range 0..1.,
+                                            360);
     SetCompassDirectionRaw(newDirection, newDeviation);
   }
 
@@ -1588,7 +1056,7 @@ public:
       InitializeGPS();
       Lum::OS::display->AddTimer(refreshTime,0,refreshTimer);
       Lum::OS::display->AddTimer(reconnectTime,0,reconnectCheckTimer);
-      RefreshTracks();
+      RefreshTracks(tracks);
       LoadConfig();
     }
     else if (model==GetClosedAction() &&
@@ -1638,21 +1106,33 @@ public:
     else if (model==deleteTrackAction &&
              deleteTrackAction->IsFinished() &&
              deleteTrackAction->IsEnabled()) {
-      DeleteTrack(tracks->GetEntry(trackSelection->GetLine()).GetName());
+      if (DeleteTrack(GetWindow(),
+                      tracks->GetEntry(trackSelection->GetLine()).GetName())) {
+        RefreshTracks(tracks);
+      }
     }
-    else if (model==exportTrackAction &&
-             exportTrackAction->IsFinished() &&
-             exportTrackAction->IsEnabled()) {
-      ExportTrack(tracks->GetEntry(trackSelection->GetLine()).GetName());
+    else if (model==exportTrackKMLAction &&
+             exportTrackKMLAction->IsFinished() &&
+             exportTrackKMLAction->IsEnabled()) {
+      ExportTrackToKML(GetWindow(),
+                       tracks->GetEntry(trackSelection->GetLine()).GetName());
+    }
+    else if (model==exportTrackGPXAction &&
+             exportTrackGPXAction->IsFinished() &&
+             exportTrackGPXAction->IsEnabled()) {
+      ExportTrackToGPX(GetWindow(),
+                       tracks->GetEntry(trackSelection->GetLine()).GetName());
     }
     else if (model==trackSelection) {
       if (trackSelection->HasSelection()) {
         deleteTrackAction->Enable();
-        exportTrackAction->Enable();
+        exportTrackKMLAction->Enable();
+        exportTrackGPXAction->Enable();
       }
       else {
         deleteTrackAction->Disable();
-        exportTrackAction->Disable();
+        exportTrackKMLAction->Disable();
+        exportTrackGPXAction->Disable();
       }
     }
     else if (model==settingsAction &&
@@ -1717,27 +1197,33 @@ static void cb(struct gps_data_t* data, char * buf, size_t len, int level)
     gpsData.SetFix(data->status!=STATUS_NO_FIX);
   }
 
-  if ((data->set & LATLON_SET)  && !isnan(data->fix.latitude)) {
+  if ((data->set & LATLON_SET)  &&
+      !isnan(data->fix.latitude)) {
     gpsData.SetLatitude(data->fix.latitude,data->fix.epy);
   }
 
-  if ((data->set & LATLON_SET)  && !isnan(data->fix.longitude)) {
+  if ((data->set & LATLON_SET)  &&
+      !isnan(data->fix.longitude)) {
     gpsData.SetLongitude(data->fix.longitude,data->fix.epx);
   }
 
-  if ((data->set & ALTITUDE_SET) && !isnan(data->fix.altitude)) {
+  if ((data->set & ALTITUDE_SET) &&
+      !isnan(data->fix.altitude)) {
     gpsData.SetAltitude(data->fix.altitude,data->fix.epv);
   }
 
-  if ((data->set & SPEED_SET) && !isnan(data->fix.speed)) {
+  if ((data->set & SPEED_SET) &&
+       !isnan(data->fix.speed)) {
     gpsData.SetSpeed(data->fix.speed,data->fix.eps); // m/s
   }
 
-  if ((data->set & TRACK_SET) && !isnan(data->fix.track)) {
+  if ((data->set & TRACK_SET) &&
+      !isnan(data->fix.track)) {
     gpsData.SetTrack(data->fix.track,data->fix.epd);
   }
 
-  if ((data->set & CLIMB_SET) && !isnan(data->fix.climb)) {
+  if ((data->set & CLIMB_SET) &&
+      !isnan(data->fix.climb)) {
     gpsData.SetClimb(data->fix.climb,data->fix.epc);
   }
 
