@@ -380,7 +380,8 @@ void TimeLine::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
   if (model==hAdjustment->GetTopModel()) {
     Redraw();
   }
-  else if (model==this->model && !this->model->IsNull()) {
+  else if (model==this->model &&
+           !this->model->IsNull()) {
     Lum::Base::SystemTime now;
     Entry                 entry;
 
@@ -390,29 +391,30 @@ void TimeLine::Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
 
     data[now]=entry;
 
-    if (isnan(entry.value)) {
+    if (isnan(entry.value) ||
+        isinf(entry.value)) {
       std::cout << "New value is not a number, skipping" << std::endl;
       return;
     }
 
     if (top==0 && bottom==0) {
       if (entry.value>=0) {
-        top=entry.value*(1.0+borderOffset);
-        bottom=entry.value*(1.0-borderOffset);
+        top=entry.value+borderOffset;
+        bottom=entry.value-borderOffset;
       }
       else {
-        top=entry.value*(1.0-borderOffset);
-        bottom=entry.value*(1.0+borderOffset);
+        top=entry.value-borderOffset;
+        bottom=entry.value+borderOffset;
       }
     }
     else {
       if (entry.value>=0) {
-        top=std::max(top,entry.value*(1.0+borderOffset));
-        bottom=std::min(bottom,entry.value*(1.0-borderOffset));
+        top=std::max(top,entry.value+borderOffset);
+        bottom=std::min(bottom,entry.value-borderOffset);
       }
       else {
-        top=std::max(top,entry.value*(1.0-borderOffset));
-        bottom=std::min(bottom,entry.value*(1.0+borderOffset));
+        top=std::max(top,entry.value-borderOffset);
+        bottom=std::min(bottom,entry.value+borderOffset);
       }
     }
 
